Attribute VB_Name = "UndoRedoStacksManagement"
Option Explicit

Private Type Stack
    strData() As String
    intIndex As Integer
    intMaxSize As Integer
End Type

Private UndoStack As Stack
Private RedoStack As Stack

'Err constants
Public Const errUndoStackEmpty = 101 + vbObjectError + 512
Public Const errRedoStackEmpty = 102 + vbObjectError + 512
Public Const errStackAlreadyInitialised = 103 + vbObjectError + 512
Public Const errInvalidParameter = 104 + vbObjectError + 512
Public Const errStacksNotIntialised = 105 + vbObjectError + 512


Public Sub InitialiseUndoSystem(intMaxSize As Integer)

    If UndoStack.intMaxSize <> 0 Or RedoStack.intMaxSize <> 0 Then
        Err.Raise errStackAlreadyInitialised, "Undo System", "Undo System has already been initialised"
    ElseIf intMaxSize < 1 Then
        Err.Raise errInvalidParameter, "Undo System", "Maximum size must be positive"
    Else
        'Intialise Undo Stack
        UndoStack.intIndex = 0
        UndoStack.intMaxSize = intMaxSize
        
        'Intialise Undo Stack
        RedoStack.intIndex = 0
        RedoStack.intMaxSize = intMaxSize
    End If
        
    
End Sub
    
Public Sub ReInitialiseUndoSystem()

    If UndoStack.intMaxSize = 0 Then
        Err.Raise errStacksNotIntialised, "Undo System", "Undo Sytem not initialised"
    Else
        'Undo stack
        UndoStack.intIndex = 0
        Erase UndoStack.strData
        
        'Redo Stack
        RedoStack.intIndex = 0
        Erase RedoStack.strData
    End If
        
End Sub

    
Public Sub AddUndoSnapshot(strSnapshot As String, Optional blnDontFlush As Boolean)
    Dim intCounter As Long
    
    If UndoStack.intIndex = 0 Then
        'if stack is blank then dimension it
        UndoStack.intIndex = 1
        ReDim UndoStack.strData(1 To 1)
    ElseIf UndoStack.intIndex < UndoStack.intMaxSize Then
        'If the stack is not full then just add an item
        UndoStack.intIndex = UndoStack.intIndex + 1
        ReDim Preserve UndoStack.strData(1 To UndoStack.intIndex)
        UndoStack.strData(UndoStack.intIndex) = strSnapshot
    Else
        'move all the items down to create room at the top
        For intCounter = 2 To UndoStack.intMaxSize
            UndoStack.strData(intCounter - 1) = UndoStack.strData(intCounter)
        Next
    End If
    
    'Place latest snapshot at the top
    UndoStack.strData(UndoStack.intIndex) = strSnapshot
    
    'Flush redo stack
    If Not blnDontFlush Then
        RedoStack.intIndex = 0
        Erase RedoStack.strData
    End If
    
    
End Sub

Public Sub AddRedoSnapshot(strSnapshot As String)
    Dim intCounter As Long
    
    If RedoStack.intIndex = 0 Then
        'if stack is blank then dimension it
        RedoStack.intIndex = 1
        ReDim RedoStack.strData(1 To 1)
    ElseIf RedoStack.intIndex < RedoStack.intMaxSize Then
        'If the stack is not full then just add an item
        RedoStack.intIndex = RedoStack.intIndex + 1
        ReDim Preserve RedoStack.strData(1 To RedoStack.intIndex)
        RedoStack.strData(RedoStack.intIndex) = strSnapshot
    Else
        'move all the items down to create room at the top
        For intCounter = 2 To RedoStack.intMaxSize
            RedoStack.strData(intCounter - 1) = RedoStack.strData(intCounter)
        Next
    End If
    
    'Place latest snapshot at the top
    RedoStack.strData(RedoStack.intIndex) = strSnapshot
    
End Sub


Public Function getUndoSnapShotsAvailable() As Integer
    getUndoSnapShotsAvailable = UndoStack.intIndex
End Function

Public Function getRedoSnapShotsAvailable() As Integer
    getRedoSnapShotsAvailable = RedoStack.intIndex
End Function


Public Function GetUndoSnapshot(strRedoSnapshot As String) As String
    
    If UndoStack.intIndex = 0 Then
        Err.Raise errUndoStackEmpty, "Undo System", "No Undo Snapshots Available"
    Else
        'Place current text snapshot on redo stack
        Call AddRedoSnapshot(strRedoSnapshot)
        
        'Pop top-most item
        GetUndoSnapshot = UndoStack.strData(UndoStack.intIndex)
        UndoStack.intIndex = UndoStack.intIndex - 1
        If UndoStack.intIndex = 0 Then
            Erase UndoStack.strData
        Else
            ReDim Preserve UndoStack.strData(1 To UndoStack.intIndex)
        End If
    
    End If
        
End Function

Public Function GetRedoSnapshot(strUndoSnapshot As String) As String
    
    If RedoStack.intIndex = 0 Then
        Err.Raise errRedoStackEmpty, "Undo System", "No Redo Snapshots Available"
    Else
        'Place current text snapshot on undo stack
        Call AddUndoSnapshot(strUndoSnapshot, True)
        
        'Pop top-most item
        GetRedoSnapshot = RedoStack.strData(RedoStack.intIndex)
        RedoStack.intIndex = RedoStack.intIndex - 1
        If RedoStack.intIndex = 0 Then
            Erase RedoStack.strData
        Else
            ReDim Preserve UndoStack.strData(1 To UndoStack.intIndex)
        End If
    
    End If
        
End Function

